@@ -839,7 +839,7 @@ This will become a hard error in a future Meson release.''')
                 self.external_deps.append(extpart)
                 # Deps of deps.
                 self.add_deps(dep.ext_deps)
-            elif isinstance(dep, dependencies.ExternalDependency):
+            elif isinstance(dep, dependencies.Dependency):
                 self.external_deps.append(dep)
                 self.process_sourcelist(dep.get_sources())
             elif isinstance(dep, BuildTarget):
@@ -1776,7 +1776,7 @@ class CustomTargetIndex:
 
     def __repr__(self):
         return '<CustomTargetIndex: {!r}[{}]>'.format(
-            self.target, self.target.output.index(self.output))
+            self.target, self.target.get_outputs().index(self.output))
 
     def get_outputs(self):
         return [self.output]
