@@ -16,6 +16,7 @@
 # Custom logic for several other packages are in separate files.
 
 import os
+import re
 import sys
 import stat
 import shlex
@@ -207,6 +208,7 @@ class ConfigToolDependency(ExternalDependency):
 
     tools = None
     tool_name = None
+    __strip_version = re.compile(r'^[0-9.]*')
 
     def __init__(self, name, environment, language, kwargs):
         super().__init__('config-tool', environment, language, kwargs)
@@ -222,6 +224,15 @@ class ConfigToolDependency(ExternalDependency):
             return
         self.version = version
 
+    def _sanitize_version(self, version):
+        """Remove any non-numeric, non-point version suffixes."""
+        m = self.__strip_version.match(version)
+        if m:
+            # Ensure that there isn't a trailing '.', such as an input like
+            # `1.2.3.git-1234`
+            return m.group(0).rstrip('.')
+        return version
+
     @classmethod
     def factory(cls, name, environment, language, kwargs, tools, tool_name):
         """Constructor for use in dependencies that can be found multiple ways.
@@ -259,7 +270,7 @@ class ConfigToolDependency(ExternalDependency):
             if p.returncode != 0:
                 continue
 
-            out = out.strip()
+            out = self._sanitize_version(out.strip())
             # Some tools, like pcap-config don't supply a version, but also
             # dont fail with --version, in that case just assume that there is
             # only one verison and return it.
@@ -470,6 +481,7 @@ class PkgConfigDependency(ExternalDependency):
                                       (self.name, out))
         self.link_args = []
         libpaths = []
+        static_libs_notfound = []
         for lib in self._convert_mingw_paths(shlex.split(out)):
             # If we want to use only static libraries, we have to look for the
             # file ourselves instead of depending on the compiler to find it
@@ -479,12 +491,18 @@ class PkgConfigDependency(ExternalDependency):
                 if lib.startswith('-L'):
                     libpaths.append(lib[2:])
                     continue
-                elif lib.startswith('-l') and libpaths:
+                # FIXME: try to handle .la files in static mode too?
+                elif lib.startswith('-l'):
                     args = self.compiler.find_library(lib[2:], self.env, libpaths, libtype='static')
                     if not args or len(args) < 1:
-                        raise DependencyException('Static library not found for {!r}'
-                                                  ''.format(lib[2:]))
-                    lib = args[0]
+                        if lib in static_libs_notfound:
+                            continue
+                        mlog.warning('Static library {!r} not found for dependency {!r}, may '
+                                     'not be statically linked'.format(lib[2:], self.name))
+                        static_libs_notfound.append(lib)
+                    else:
+                        # Replace -l arg with full path to static library
+                        lib = args[0]
             elif lib.endswith(".la"):
                 shared_libname = self.extract_libtool_shlib(lib)
                 shared_lib = os.path.join(os.path.dirname(lib), shared_libname)
@@ -498,6 +516,11 @@ class PkgConfigDependency(ExternalDependency):
                 lib = shared_lib
                 self.is_libtool = True
             self.link_args.append(lib)
+        # Add all -Lbar args if we have -lfoo args in link_args
+        if static_libs_notfound:
+            # Order of -L flags doesn't matter with ld, but it might with other
+            # linkers such as MSVC, so prepend them.
+            self.link_args = ['-L' + lp for lp in libpaths] + self.link_args
 
     def get_pkgconfig_variable(self, variable_name, kwargs):
         options = ['--variable=' + variable_name, self.name]
@@ -753,6 +776,17 @@ class ExternalProgram:
     def get_name(self):
         return self.name
 
+class NonExistingExternalProgram(ExternalProgram):
+
+    def __init__(self):
+        super().__init__(name = 'nonexistingprogram', silent = True)
+
+    def __repr__(self):
+        r = '<{} {!r} -> {!r}>'
+        return r.format(self.__class__.__name__, self.name, self.command)
+
+    def found(self):
+        return False
 
 class ExternalLibrary(ExternalDependency):
     def __init__(self, name, link_args, environment, language, silent=False):
