@@ -12,10 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import shutil
+
 from os import path
 from .. import coredata, mesonlib, build
 from ..mesonlib import MesonException
-import sys
+from . import ModuleReturnValue
+from . import ExtensionModule
+from ..interpreterbase import permittedKwargs
 
 PRESET_ARGS = {
     'glib': [
@@ -43,8 +47,16 @@ PRESET_ARGS = {
     ]
 }
 
-class I18nModule:
+class I18nModule(ExtensionModule):
 
+    @staticmethod
+    def _get_data_dirs(state, dirs):
+        """Returns source directories of relative paths"""
+        src_dir = path.join(state.environment.get_source_dir(), state.subdir)
+        return [path.join(src_dir, d) for d in dirs]
+
+    @permittedKwargs({'languages', 'data_dirs', 'preset', 'args', 'po_dir', 'type',
+                      'input', 'output', 'install', 'install_dir'})
     def merge_file(self, state, args, kwargs):
         podir = kwargs.pop('po_dir', None)
         if not podir:
@@ -53,19 +65,32 @@ class I18nModule:
 
         file_type = kwargs.pop('type', 'xml')
         VALID_TYPES = ('xml', 'desktop')
-        if not file_type in VALID_TYPES:
+        if file_type not in VALID_TYPES:
             raise MesonException('i18n: "{}" is not a valid type {}'.format(file_type, VALID_TYPES))
 
-        kwargs['command'] = ['msgfmt', '--' + file_type,
-                             '--template', '@INPUT@', '-d', podir, '-o', '@OUTPUT@']
-        return build.CustomTarget(kwargs['output'] + '_merge', state.subdir, kwargs)
+        datadirs = self._get_data_dirs(state, mesonlib.stringlistify(kwargs.pop('data_dirs', [])))
+        datadirs = '--datadirs=' + ':'.join(datadirs) if datadirs else None
 
+        command = state.environment.get_build_command() + [
+            '--internal', 'msgfmthelper',
+            '@INPUT@', '@OUTPUT@', file_type, podir
+        ]
+        if datadirs:
+            command.append(datadirs)
+
+        kwargs['command'] = command
+        ct = build.CustomTarget(kwargs['output'] + '_merge', state.subdir, state.subproject, kwargs)
+        return ModuleReturnValue(ct, [ct])
+
+    @permittedKwargs({'po_dir', 'data_dirs', 'type', 'languages', 'args', 'preset', 'install'})
     def gettext(self, state, args, kwargs):
         if len(args) != 1:
             raise coredata.MesonException('Gettext requires one positional argument (package name).')
+        if not shutil.which('xgettext'):
+            raise coredata.MesonException('Can not do gettext because xgettext is not installed.')
         packagename = args[0]
         languages = mesonlib.stringlistify(kwargs.get('languages', []))
-        datadirs = mesonlib.stringlistify(kwargs.get('data_dirs', []))
+        datadirs = self._get_data_dirs(state, mesonlib.stringlistify(kwargs.get('data_dirs', [])))
         extra_args = mesonlib.stringlistify(kwargs.get('args', []))
 
         preset = kwargs.pop('preset', None)
@@ -81,37 +106,42 @@ class I18nModule:
         datadirs = '--datadirs=' + ':'.join(datadirs) if datadirs else None
         extra_args = '--extra-args=' + '@@'.join(extra_args) if extra_args else None
 
-        potargs = [state.environment.get_build_command(), '--internal', 'gettext', 'pot', pkg_arg]
+        potargs = state.environment.get_build_command() + ['--internal', 'gettext', 'pot', pkg_arg]
         if datadirs:
             potargs.append(datadirs)
         if extra_args:
             potargs.append(extra_args)
-        pottarget = build.RunTarget(packagename + '-pot', sys.executable, potargs, [], state.subdir)
+        pottarget = build.RunTarget(packagename + '-pot', potargs[0], potargs[1:], [], state.subdir, state.subproject)
 
-        gmoargs = [state.environment.get_build_command(), '--internal', 'gettext', 'gen_gmo']
+        gmoargs = state.environment.get_build_command() + ['--internal', 'gettext', 'gen_gmo']
         if lang_arg:
             gmoargs.append(lang_arg)
-        gmotarget = build.RunTarget(packagename + '-gmo', sys.executable, gmoargs, [], state.subdir)
+        gmotarget = build.RunTarget(packagename + '-gmo', gmoargs[0], gmoargs[1:], [], state.subdir, state.subproject)
 
-        updatepoargs = [state.environment.get_build_command(), '--internal', 'gettext', 'update_po', pkg_arg]
+        updatepoargs = state.environment.get_build_command() + ['--internal', 'gettext', 'update_po', pkg_arg]
         if lang_arg:
             updatepoargs.append(lang_arg)
         if datadirs:
             updatepoargs.append(datadirs)
         if extra_args:
             updatepoargs.append(extra_args)
-        updatepotarget = build.RunTarget(packagename + '-update-po', sys.executable, updatepoargs, [], state.subdir)
+        updatepotarget = build.RunTarget(packagename + '-update-po', updatepoargs[0], updatepoargs[1:], [], state.subdir, state.subproject)
 
-        script = [sys.executable,  state.environment.get_build_command()]
-        args = ['--internal', 'gettext', 'install',
-                '--subdir=' + state.subdir,
-                '--localedir=' + state.environment.coredata.get_builtin_option('localedir'),
-                pkg_arg]
-        if lang_arg:
-            args.append(lang_arg)
-        iscript = build.RunScript(script, args)
+        targets = [pottarget, gmotarget, updatepotarget]
 
-        return [pottarget, gmotarget, iscript, updatepotarget]
+        install = kwargs.get('install', True)
+        if install:
+            script = state.environment.get_build_command()
+            args = ['--internal', 'gettext', 'install',
+                    '--subdir=' + state.subdir,
+                    '--localedir=' + state.environment.coredata.get_builtin_option('localedir'),
+                    pkg_arg]
+            if lang_arg:
+                args.append(lang_arg)
+            iscript = build.RunScript(script, args)
+            targets.append(iscript)
+
+        return ModuleReturnValue(None, targets)
 
 def initialize():
     return I18nModule()
